const parseJSONScript = (document2, id) => {
  const script = document2.getElementById(id);
  try {
    return JSON.parse(script.innerHTML);
  } catch {
    return null;
  }
};
const renderHtmlTemplate = (config, targetElement, templateName, context = {}, renderMethod = "after") => {
  const interpolatedTemplate = Object.entries(context).reduce((output, value) => {
    const [k, v] = value;
    return output.replace(new RegExp(`{{ ${k} }}`, "g"), v);
  }, config.templates[templateName]);
  const templateDocument = new DOMParser().parseFromString(interpolatedTemplate, "text/html");
  const templateElement = templateDocument.querySelector("body > *");
  targetElement[renderMethod](templateElement);
  return templateElement;
};
const GET = "get";
const API_METHODS = {
  check_balance: {
    http_method: GET,
    path: "/balance.json"
  },
  preauth: {
    http_method: GET,
    path: "/preauth.json"
  }
};
const getUrl = (endpoint, method) => {
  return `${endpoint}${API_METHODS[method].path}`;
};
const getHttpMethod = (method) => {
  return API_METHODS[method].http_method;
};
const getQueryParams = (authentication, params) => {
  return Object.assign({}, authentication, params);
};
const buildQueryString = (params) => {
  const queryString = Object.keys(params).map((key) => `${encodeURIComponent(key)}=${encodeURIComponent(params[key])}`).join("&");
  return `?${queryString}`;
};
class ApiClient {
  constructor({ authentication, endpoint }) {
    this.authentication = authentication;
    this.endpoint = endpoint;
  }
  execute({ method, params, onSuccess, onFailure }) {
    const url = getUrl(this.endpoint, method);
    const httpMethod = getHttpMethod(method);
    const queryParams = getQueryParams(this.authentication, params);
    return fetch(url + buildQueryString(queryParams), {
      method: httpMethod,
      mode: "cors",
      headers: {
        "Content-Type": "application/json; charset=utf-8"
      }
    }).then((response) => response.json().then((json) => {
      if (json && json.response) {
        if (json.response.status === "OK") {
          onSuccess && onSuccess(json.response);
          return;
        } else {
          onFailure && onFailure(json.response);
          return;
        }
      }
      if (response.status === 400 || response.status === 422 || response.status === 500) {
        onFailure && onFailure(json);
        return;
      }
      onSuccess && onSuccess(json);
    }));
  }
  checkBalance({ number, pin, onSuccess, onFailure }) {
    return this.execute({
      method: "check_balance",
      params: {
        voucher_number: number,
        security_code: pin
      },
      onSuccess,
      onFailure
    });
  }
  preauth({ number, pin, amount, onSuccess, onFailure }) {
    return this.execute({
      method: "preauth",
      params: {
        voucher_number: number,
        security_code: pin,
        voucher_amount: amount
      },
      onSuccess,
      onFailure
    });
  }
}
const STEP_CONTACT_INFORMATION = "contact_information";
const STEP_SHIPPING_METHOD = "shipping_method";
const STEP_PAYMENT_METHOD = "payment_method";
const SELECTOR_BALANCE_CHECKER_FORM = '[data-givex-balance-checker="form"]';
const SELECTOR_BALANCE_CHECKER_NUMBER = '[data-givex-balance-checker="number"]';
const SELECTOR_BALANCE_CHECKER_PIN = '[data-givex-balance-checker="pin"]';
const SELECTOR_BALANCE_CHECKER_RESULT = '[data-givex-balance-checker="result"]';
const SELECTOR_BALANCE_CHECKER_SUBMIT = '[data-givex-balance-checker="submit"]';
const SELECTOR_DISCOUNT_INPUT = '[data-discount-field="true"]';
const SELECTOR_FIELDSET = ".fieldset";
const SELECTOR_FORM = "form";
const SELECTOR_REDUCTION_FORM_WRAPPER = "[data-reduction-form]";
const SELECTOR_SECURITY_CODE_INPUT = '[name="checkout[security_code]"]';
const SELECTOR_SUBMIT_BUTTON = '[type="submit"]';
const SECURITY_CODE_POLICY_IS_REQUIRED = "security_code_is_required";
const SECURITY_CODE_POLICY_IS_OPTIONAL = "security_code_is_optional";
class CheckoutForm {
  constructor(formWrapperElement, api, config) {
    this.formWrapperElement = formWrapperElement;
    this.api = api;
    this.config = config;
    this.potentialGivexCard = false;
    this.initialise();
  }
  initialise() {
    this.debug("initialise()");
    const { formWrapperElement } = this;
    this.inputElement = formWrapperElement.querySelector(SELECTOR_DISCOUNT_INPUT);
    this.fieldsetElement = this.inputElement.closest(SELECTOR_FIELDSET);
    this.formElement = this.inputElement.closest(SELECTOR_FORM);
    this.submitElement = formWrapperElement.querySelector(SELECTOR_SUBMIT_BUTTON);
    if (this.config.security_code_policy === SECURITY_CODE_POLICY_IS_REQUIRED || this.config.security_code_policy === SECURITY_CODE_POLICY_IS_OPTIONAL) {
      this.renderSecurityCode();
      this.securityCodeInputElement = formWrapperElement.querySelector(SELECTOR_SECURITY_CODE_INPUT);
    }
    this.inputElement.addEventListener("input", this.handleInput.bind(this));
    this.inputElement.addEventListener("keypress", this.handleKeypress.bind(this));
    this.formElement.addEventListener("submit", this.handleSubmit.bind(this));
    this.submitElement.addEventListener("click", this.handleClick.bind(this));
    formWrapperElement.dataset.givex = "true";
  }
  renderSecurityCode() {
    const { config } = this;
    renderHtmlTemplate(config, this.fieldsetElement, "checkout_security_code");
  }
  handleInput(e) {
    this.debug("handleInput()", e);
    this.potentialGivexCard = this.isPotentialGivexCard(this.inputElement.value);
    this.debug("potentialGivexCard", this.potentialGivexCard);
    this.formWrapperElement.classList.toggle("is-potential-givex-card", this.potentialGivexCard);
  }
  handleKeypress(e) {
    this.debug("handleKeypress()", e);
    if (e.keyCode === 13) {
      this.potentialGivexCard = this.isPotentialGivexCard(this.inputElement.value);
      this.debug("potentialGivexCard", this.potentialGivexCard);
      if (this.potentialGivexCard) {
        e.stopImmediatePropagation();
        this.handleSubmit();
      }
    }
  }
  handleClick(e) {
    this.debug("handleClick()", e);
    if (this.potentialGivexCard) {
      e.stopImmediatePropagation();
      this.handleSubmit();
    }
  }
  handleSubmit(e) {
    this.debug("handleSubmit()", e);
    const { formWrapperElement, potentialGivexCard, inputElement, securityCodeInputElement, submitElement, api, config } = this;
    if (!potentialGivexCard) {
      return true;
    }
    if (formWrapperElement.dataset.forceSubmit === "true") {
      return true;
    }
    e && e.preventDefault();
    e && e.stopPropagation();
    if (securityCodeInputElement && securityCodeInputElement.value.trim().length === 0) {
      securityCodeInputElement.focus();
      return false;
    }
    submitElement.classList.add("btn--loading");
    submitElement.disabled = true;
    const number = inputElement.value;
    const pin = securityCodeInputElement ? securityCodeInputElement.value : null;
    const amount = config.checkout.total_price;
    api.preauth({
      number,
      pin,
      amount,
      onSuccess: this.handlePreauthSuccess.bind(this),
      onFailure: this.handlePreauthFailure.bind(this)
    });
  }
  handlePreauthSuccess(result) {
    this.debug("handlePreauthSuccess()", result);
    const { inputElement } = this;
    const hiddenInput = document.createElement("input");
    hiddenInput.type = "hidden";
    hiddenInput.name = "checkout[reduction_code]";
    hiddenInput.value = result.voucher_number;
    inputElement.after(hiddenInput);
    inputElement.disabled = true;
    if (this.securityCodeInputElement) {
      this.securityCodeInputElement.disabled = true;
    }
    this.handlePreauthComplete();
  }
  handlePreauthFailure(error) {
    this.debug("handlePreauthFailure()", error);
    this.handlePreauthComplete();
  }
  handlePreauthComplete() {
    this.debug("handlePreauthComplete()");
    const { formWrapperElement, formElement } = this;
    formWrapperElement.dataset.forceSubmit = "true";
    formElement.submit();
  }
  isPotentialGivexCard(value) {
    this.debug("isPotentialGivexCard()", value);
    const cleanValue = value.replace(/\D/g, "");
    return cleanValue.length >= this.config.card_code_length;
  }
  debug(...args) {
    if (!this.config.debug) {
      return;
    }
    console.log("[Givex CheckoutForm]", ...args);
  }
}
const SUPPORTED_STEPS = [
  STEP_CONTACT_INFORMATION,
  STEP_SHIPPING_METHOD,
  STEP_PAYMENT_METHOD
];
class Checkout {
  constructor(document2, Shopify, api, config) {
    this.document = document2;
    this.Shopify = Shopify;
    this.api = api;
    this.config = config;
    this.initialise();
  }
  initialise() {
    this.debug("initialise()");
    const { document: document2, Shopify, api, config } = this;
    if (!Shopify || !Shopify.Checkout || !SUPPORTED_STEPS.includes(Shopify.Checkout.step)) {
      return;
    }
    this.debug("on supported step");
    const handlePageChange = () => {
      document2.querySelectorAll(SELECTOR_REDUCTION_FORM_WRAPPER).forEach((formWrapperElement) => {
        if (formWrapperElement.dataset.givex === "true") {
          return;
        }
        new CheckoutForm(formWrapperElement, api, config);
      });
    };
    document2.addEventListener("page:load", handlePageChange);
    document2.addEventListener("page:change", handlePageChange);
    handlePageChange();
  }
  debug(...args) {
    if (!this.config.debug) {
      return;
    }
    console.log("[Givex Checkout]", ...args);
  }
}
class BalanceCheckerForm {
  constructor(formElement, api, config) {
    this.formElement = formElement;
    this.api = api;
    this.config = config;
    this.initialise();
  }
  initialise() {
    this.debug("initialise()");
    const { formElement } = this;
    this.numberElement = formElement.querySelector(SELECTOR_BALANCE_CHECKER_NUMBER);
    this.pinElement = formElement.querySelector(SELECTOR_BALANCE_CHECKER_PIN);
    this.resultElement = formElement.querySelector(SELECTOR_BALANCE_CHECKER_RESULT);
    this.submitElement = formElement.querySelector(SELECTOR_BALANCE_CHECKER_SUBMIT);
    this.formElement.addEventListener("submit", this.handleSubmit.bind(this));
    formElement.dataset.givex = "true";
  }
  handleSubmit(e) {
    this.debug("handleSubmit()", e);
    const { numberElement, pinElement, submitElement, resultElement, api, config } = this;
    e.preventDefault();
    e.stopPropagation();
    if (config.security_code_policy === SECURITY_CODE_POLICY_IS_REQUIRED && pinElement && pinElement.value.trim().length === 0) {
      pinElement.focus();
      return false;
    }
    submitElement.classList.add("btn--loading");
    submitElement.disabled = true;
    this.resultElement = renderHtmlTemplate(config, resultElement, "balance_checker_loading", {}, "replaceWith");
    const number = numberElement.value;
    const pin = pinElement ? pinElement.value : null;
    api.checkBalance({
      number,
      pin,
      onSuccess: this.handleBalanceCheckSuccess.bind(this),
      onFailure: this.handleBalanceCheckFailure.bind(this)
    });
  }
  handleBalanceCheckSuccess(result) {
    this.debug("handleBalanceCheckSuccess", result);
    const { config, resultElement } = this;
    this.resultElement = renderHtmlTemplate(config, resultElement, "balance_checker_success", {
      message: result.message
    }, "replaceWith");
    this.handleBalanceCheckComplete();
  }
  handleBalanceCheckFailure(error) {
    this.debug("handleBalanceCheckFailure", error);
    const { config, resultElement } = this;
    this.resultElement = renderHtmlTemplate(config, resultElement, "balance_checker_error", {
      message: error.message
    }, "replaceWith");
    this.handleBalanceCheckComplete();
  }
  handleBalanceCheckComplete() {
    const { submitElement } = this;
    submitElement.classList.remove("btn--loading");
    submitElement.disabled = false;
  }
  debug(...args) {
    if (!this.config.debug) {
      return;
    }
    console.log("[Givex BalanceCheckerForm]", ...args);
  }
}
class BalanceChecker {
  constructor(document2, api, config) {
    this.document = document2;
    this.api = api;
    this.config = config;
    this.initialise();
  }
  initialise() {
    this.debug("initialise()");
    const { document: document2, api, config } = this;
    document2.querySelectorAll(SELECTOR_BALANCE_CHECKER_FORM).forEach((formElement) => {
      if (formElement.dataset.givex === "true") {
        return;
      }
      new BalanceCheckerForm(formElement, api, config);
    });
  }
  debug(...args) {
    if (!this.config.debug) {
      return;
    }
    console.log("[Givex Balance Checker]", ...args);
  }
}
class Givex {
  constructor(document2, Shopify, config = {}) {
    const api = new ApiClient(config);
    this.api = api;
    this.checkout = new Checkout(document2, Shopify, api, config);
    this.balanceChecker = new BalanceChecker(document2, api, config);
  }
}
const initialise = () => {
  if (!window || !document) {
    return;
  }
  const Shopify = window.Shopify;
  const config = parseJSONScript(document, "givex-config");
  window.givex = new Givex(document, Shopify, config);
};
initialise();
